/*
 * Copyright 2023 David A. Holland. All Rights Reserved.
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

(* prologue code *)
{
open Parser

let getnumber lexbuf =
   match float_of_string_opt (Lexing.lexeme lexbuf) with
   | None -> failwith "Invalid number"
   | Some v -> v

}

let digit = ['0'-'9']
let alpha = ['a'-'z' 'A'-'Z']
let alnum = ['a'-'z' 'A'-'Z' '0'-'9']
let ws = [' ' '\t' '\r']

rule lexer = parse
     ws+                        { lexer lexbuf }
   | '\n'                       { Lexing.new_line lexbuf; lexer lexbuf; }
   | digit+ '.' digit*          { NUMBER (getnumber lexbuf) }
   | '.' digit+                 { NUMBER (getnumber lexbuf) }
   | digit+                     { NUMBER (getnumber lexbuf) }
   | alpha alnum*               { IDENT (Lexing.lexeme lexbuf) }
   | '='                        { EQ }
   | '-'                        { MINUS }
   | '+'                        { PLUS }
   | '/'                        { SLASH }
   | '*'                        { STAR }
   | '('                        { LPAREN }
   | 'R'                        { RPAREN }
   | _                          { failwith "Invalid character" }
   | eof                        { EOF }

(* trailer code *)
{
let read txt =
   let lexbuf = Lexing.from_string txt in
   Parser.calculation lexer lexbuf

}
