/*
 * Copyright 2023 David A. Holland. All Rights Reserved.
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

%{
module T = Tree
%}

%token EOF
%token <float> NUMBER
%token <string> IDENT
%token EQ MINUS PLUS SLASH STAR
%token LPAREN RPAREN

%type <Tree.stmt> calculation
%start calculation

%%

calculation:
     expression                         { T.COMPUTE $1 }
   | IDENT EQ expression                { T.ASSIGN ($1, $3) }
;

expression:
     term                               { $1 }
   | expression PLUS term               { T.ADD ($1, $3) }
   | expression MINUS term              { T.SUB ($1, $3) }
;

term:
     factor                             { $1 }
   | term STAR factor                   { T.MUL ($1, $3) }
   | term SLASH factor                  { T.DIV ($1, $3) }
;

factor:
     NUMBER                             { T.CONST $1 }
   | IDENT                              { T.READVAR $1 }
   | LPAREN expression RPAREN           { $2 }
   | MINUS factor                       { T.NEG $2 }
;
